@@ -1,6 +1,4 @@
 /*
- * $Id: file.c,v 1.5 2005/07/03 07:00:56 daniels Exp $
- *
  * Copyright © 2002 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -34,7 +32,7 @@ XcursorImageCreate (int width, int height)
     image = malloc (sizeof (XcursorImage) +
 		    width * height * sizeof (XcursorPixel));
     if (!image)
-	return 0;
+	return NULL;
     image->version = XCURSOR_IMAGE_VERSION;
     image->pixels = (XcursorPixel *) (image + 1);
     image->size = width > height ? width : height;
@@ -47,8 +45,7 @@ XcursorImageCreate (int width, int height)
 void
 XcursorImageDestroy (XcursorImage *image)
 {
-    if (image)
-        free (image);
+    free (image);
 }
 
 XcursorImages *
@@ -56,13 +53,13 @@ XcursorImagesCreate (int size)
 {
     XcursorImages   *images;
 
-    images = malloc (sizeof (XcursorImages) + 
+    images = malloc (sizeof (XcursorImages) +
 		     size * sizeof (XcursorImage *));
     if (!images)
-	return 0;
+	return NULL;
     images->nimage = 0;
     images->images = (XcursorImage **) (images + 1);
-    images->name = 0;
+    images->name = NULL;
     return images;
 }
 
@@ -85,10 +82,10 @@ void
 XcursorImagesSetName (XcursorImages *images, const char *name)
 {
     char    *new;
-    
+
     if (!images || !name)
         return;
-    
+
     new = malloc (strlen (name) + 1);
 
     if (!new)
@@ -106,11 +103,11 @@ XcursorCommentCreate (XcursorUInt comment_type, int length)
     XcursorComment  *comment;
 
     if (length > XCURSOR_COMMENT_MAX_LEN)
-	return 0;
+	return NULL;
 
     comment = malloc (sizeof (XcursorComment) + length + 1);
     if (!comment)
-	return 0;
+	return NULL;
     comment->version = XCURSOR_COMMENT_VERSION;
     comment->comment_type = comment_type;
     comment->comment = (char *) (comment + 1);
@@ -121,8 +118,7 @@ XcursorCommentCreate (XcursorUInt comment_type, int length)
 void
 XcursorCommentDestroy (XcursorComment *comment)
 {
-    if (!comment)
-        free (comment);
+    free (comment);
 }
 
 XcursorComments *
@@ -133,7 +129,7 @@ XcursorCommentsCreate (int size)
     comments = malloc (sizeof (XcursorComments) +
 		       size * sizeof (XcursorComment *));
     if (!comments)
-	return 0;
+	return NULL;
     comments->ncomment = 0;
     comments->comments = (XcursorComment **) (comments + 1);
     return comments;
@@ -184,7 +180,7 @@ _XcursorWriteUInt (XcursorFile *file, XcursorUInt u)
 
     if (!file)
         return XcursorFalse;
-    
+
     bytes[0] = u;
     bytes[1] = u >>  8;
     bytes[2] = u >> 16;
@@ -205,8 +201,7 @@ _XcursorWriteBytes (XcursorFile *file, char *bytes, int length)
 static void
 _XcursorFileHeaderDestroy (XcursorFileHeader *fileHeader)
 {
-    if (!fileHeader)
-        free (fileHeader);
+    free (fileHeader);
 }
 
 static XcursorFileHeader *
@@ -215,11 +210,11 @@ _XcursorFileHeaderCreate (int ntoc)
     XcursorFileHeader	*fileHeader;
 
     if (ntoc > 0x10000)
-	return 0;
+	return NULL;
     fileHeader = malloc (sizeof (XcursorFileHeader) +
 			 ntoc * sizeof (XcursorFileToc));
     if (!fileHeader)
-	return 0;
+	return NULL;
     fileHeader->magic = XCURSOR_MAGIC;
     fileHeader->header = XCURSOR_FILE_HEADER_LEN;
     fileHeader->version = XCURSOR_FILE_VERSION;
@@ -237,24 +232,24 @@ _XcursorReadFileHeader (XcursorFile *file)
 
     if (!file)
         return NULL;
-    
+
     if (!_XcursorReadUInt (file, &head.magic))
-	return 0;
+	return NULL;
     if (head.magic != XCURSOR_MAGIC)
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.header))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.version))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.ntoc))
-	return 0;
+	return NULL;
     skip = head.header - XCURSOR_FILE_HEADER_LEN;
     if (skip)
 	if ((*file->seek) (file, skip, SEEK_CUR) == EOF)
-	    return 0;
+	    return NULL;
     fileHeader = _XcursorFileHeaderCreate (head.ntoc);
     if (!fileHeader)
-	return 0;
+	return NULL;
     fileHeader->magic = head.magic;
     fileHeader->header = head.header;
     fileHeader->version = head.version;
@@ -271,7 +266,7 @@ _XcursorReadFileHeader (XcursorFile *file)
     if (n != fileHeader->ntoc)
     {
 	_XcursorFileHeaderDestroy (fileHeader);
-	return 0;
+	return NULL;
     }
     return fileHeader;
 }
@@ -279,7 +274,7 @@ _XcursorReadFileHeader (XcursorFile *file)
 static XcursorUInt
 _XcursorFileHeaderLength (XcursorFileHeader *fileHeader)
 {
-    return (XCURSOR_FILE_HEADER_LEN + 
+    return (XCURSOR_FILE_HEADER_LEN +
 	    fileHeader->ntoc * XCURSOR_FILE_TOC_LEN);
 }
 
@@ -290,7 +285,7 @@ _XcursorWriteFileHeader (XcursorFile *file, XcursorFileHeader *fileHeader)
 
     if (!file || !fileHeader)
         return XcursorFalse;
-    
+
     if (!_XcursorWriteUInt (file, fileHeader->magic))
 	return XcursorFalse;
     if (!_XcursorWriteUInt (file, fileHeader->header))
@@ -312,7 +307,7 @@ _XcursorWriteFileHeader (XcursorFile *file, XcursorFileHeader *fileHeader)
 }
 
 static XcursorBool
-_XcursorSeekToToc (XcursorFile		*file, 
+_XcursorSeekToToc (XcursorFile		*file,
 		   XcursorFileHeader	*fileHeader,
 		   int			toc)
 {
@@ -428,7 +423,7 @@ _XcursorFindImageToc (XcursorFileHeader	*fileHeader,
 }
 
 static XcursorImage *
-_XcursorReadImage (XcursorFile		*file, 
+_XcursorReadImage (XcursorFile		*file,
 		   XcursorFileHeader	*fileHeader,
 		   int			toc)
 {
@@ -442,25 +437,25 @@ _XcursorReadImage (XcursorFile		*file,
         return NULL;
 
     if (!_XcursorFileReadChunkHeader (file, fileHeader, toc, &chunkHeader))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.width))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.height))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.xhot))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.yhot))
-	return 0;
+	return NULL;
     if (!_XcursorReadUInt (file, &head.delay))
-	return 0;
+	return NULL;
     /* sanity check data */
     if (head.width >= 0x10000 || head.height > 0x10000)
-	return 0;
+	return NULL;
     if (head.width == 0 || head.height == 0)
-	return 0;
+	return NULL;
     if (head.xhot > head.width || head.yhot > head.height)
-	return 0;
-    
+	return NULL;
+
     /* Create the image and initialize it */
     image = XcursorImageCreate (head.width, head.height);
     if (chunkHeader.version < image->version)
@@ -476,7 +471,7 @@ _XcursorReadImage (XcursorFile		*file,
 	if (!_XcursorReadUInt (file, p))
 	{
 	    XcursorImageDestroy (image);
-	    return 0;
+	    return NULL;
 	}
 	p++;
     }
@@ -493,7 +488,7 @@ _XcursorImageLength (XcursorImage   *image)
 }
 
 static XcursorBool
-_XcursorWriteImage (XcursorFile		*file, 
+_XcursorWriteImage (XcursorFile		*file,
 		    XcursorFileHeader	*fileHeader,
 		    int			toc,
 		    XcursorImage	*image)
@@ -513,16 +508,16 @@ _XcursorWriteImage (XcursorFile		*file,
 	return XcursorFalse;
     if (image->xhot > image->width || image->yhot > image->height)
 	return XcursorFalse;
-    
+
     /* write chunk header */
     chunkHeader.header = XCURSOR_IMAGE_HEADER_LEN;
     chunkHeader.type = XCURSOR_IMAGE_TYPE;
     chunkHeader.subtype = image->size;
     chunkHeader.version = XCURSOR_IMAGE_VERSION;
-    
+
     if (!_XcursorFileWriteChunkHeader (file, fileHeader, toc, &chunkHeader))
 	return XcursorFalse;
-    
+
     /* write extra image header fields */
     if (!_XcursorWriteUInt (file, image->width))
 	return XcursorFalse;
@@ -534,7 +529,7 @@ _XcursorWriteImage (XcursorFile		*file,
 	return XcursorFalse;
     if (!_XcursorWriteUInt (file, image->delay))
 	return XcursorFalse;
-    
+
     /* write the image */
     n = image->width * image->height;
     p = image->pixels;
@@ -548,7 +543,7 @@ _XcursorWriteImage (XcursorFile		*file,
 }
 
 static XcursorComment *
-_XcursorReadComment (XcursorFile	    *file, 
+_XcursorReadComment (XcursorFile	    *file,
 		     XcursorFileHeader	    *fileHeader,
 		     int		    toc)
 {
@@ -561,17 +556,17 @@ _XcursorReadComment (XcursorFile	    *file,
 
     /* read chunk header */
     if (!_XcursorFileReadChunkHeader (file, fileHeader, toc, &chunkHeader))
-	return 0;
+	return NULL;
     /* read extra comment header fields */
     if (!_XcursorReadUInt (file, &length))
-	return 0;
+	return NULL;
     comment = XcursorCommentCreate (chunkHeader.subtype, length);
     if (!comment)
-	return 0;
+	return NULL;
     if (!_XcursorReadBytes (file, comment->comment, length))
     {
 	XcursorCommentDestroy (comment);
-	return 0;
+	return NULL;
     }
     comment->comment[length] = '\0';
     return comment;
@@ -584,7 +579,7 @@ _XcursorCommentLength (XcursorComment	    *comment)
 }
 
 static XcursorBool
-_XcursorWriteComment (XcursorFile	    *file, 
+_XcursorWriteComment (XcursorFile	    *file,
 		      XcursorFileHeader	    *fileHeader,
 		      int		    toc,
 		      XcursorComment	    *comment)
@@ -596,24 +591,24 @@ _XcursorWriteComment (XcursorFile	    *file,
         return XcursorFalse;
 
     length = strlen (comment->comment);
-    
+
     /* sanity check data */
     if (length > XCURSOR_COMMENT_MAX_LEN)
 	return XcursorFalse;
-    
+
     /* read chunk header */
     chunkHeader.header = XCURSOR_COMMENT_HEADER_LEN;
     chunkHeader.type = XCURSOR_COMMENT_TYPE;
     chunkHeader.subtype = comment->comment_type;
     chunkHeader.version = XCURSOR_COMMENT_VERSION;
-    
+
     if (!_XcursorFileWriteChunkHeader (file, fileHeader, toc, &chunkHeader))
 	return XcursorFalse;
-    
+
     /* write extra comment header fields */
     if (!_XcursorWriteUInt (file, length))
 	return XcursorFalse;
-    
+
     if (!_XcursorWriteBytes (file, comment->comment, length))
 	return XcursorFalse;
     return XcursorTrue;
@@ -627,18 +622,18 @@ XcursorXcFileLoadImage (XcursorFile *file, int size)
     int			nsize;
     int			toc;
     XcursorImage	*image;
-    
+
     if (size < 0)
-	return 0;
+	return NULL;
     fileHeader = _XcursorReadFileHeader (file);
     if (!fileHeader)
-	return 0;
+	return NULL;
     bestSize = _XcursorFindBestSize (fileHeader, (XcursorDim) size, &nsize);
     if (!bestSize)
-	return 0;
+	return NULL;
     toc = _XcursorFindImageToc (fileHeader, bestSize, 0);
     if (toc < 0)
-	return 0;
+	return NULL;
     image = _XcursorReadImage (file, fileHeader, toc);
     _XcursorFileHeaderDestroy (fileHeader);
     return image;
@@ -653,7 +648,7 @@ XcursorXcFileLoadImages (XcursorFile *file, int size)
     XcursorImages	*images;
     int			n;
     int			toc;
-    
+
     if (!file || size < 0)
 	return NULL;
     fileHeader = _XcursorReadFileHeader (file);
@@ -676,7 +671,7 @@ XcursorXcFileLoadImages (XcursorFile *file, int size)
 	toc = _XcursorFindImageToc (fileHeader, bestSize, n);
 	if (toc < 0)
 	    break;
-	images->images[images->nimage] = _XcursorReadImage (file, fileHeader, 
+	images->images[images->nimage] = _XcursorReadImage (file, fileHeader,
 							    toc);
 	if (!images->images[images->nimage])
 	    break;
@@ -686,7 +681,7 @@ XcursorXcFileLoadImages (XcursorFile *file, int size)
     if (images->nimage != nsize)
     {
 	XcursorImagesDestroy (images);
-	images = 0;
+	images = NULL;
     }
     return images;
 }
@@ -700,13 +695,13 @@ XcursorXcFileLoadAllImages (XcursorFile *file)
     int			nimage;
     int			n;
     int			toc;
-    
+
     if (!file)
-        return 0;
-    
+        return NULL;
+
     fileHeader = _XcursorReadFileHeader (file);
     if (!fileHeader)
-	return 0;
+	return NULL;
     nimage = 0;
     for (n = 0; n < fileHeader->ntoc; n++)
     {
@@ -718,7 +713,10 @@ XcursorXcFileLoadAllImages (XcursorFile *file)
     }
     images = XcursorImagesCreate (nimage);
     if (!images)
-	return 0;
+    {
+	_XcursorFileHeaderDestroy (fileHeader);
+	return NULL;
+    }
     for (toc = 0; toc < fileHeader->ntoc; toc++)
     {
 	switch (fileHeader->tocs[toc].type) {
@@ -736,7 +734,7 @@ XcursorXcFileLoadAllImages (XcursorFile *file)
     if (images->nimage != nimage)
     {
 	XcursorImagesDestroy (images);
-	images = 0;
+	images = NULL;
     }
     return images;
 }
@@ -754,7 +752,7 @@ XcursorXcFileLoad (XcursorFile	    *file,
     XcursorComment	*comment;
     XcursorComments	*comments;
     int			toc;
-    
+
     if (!file)
         return 0;
     fileHeader = _XcursorReadFileHeader (file);
@@ -808,8 +806,8 @@ XcursorXcFileLoad (XcursorFile	    *file,
     {
 	XcursorImagesDestroy (images);
 	XcursorCommentsDestroy (comments);
-	images = 0;
-	comments = 0;
+	images = NULL;
+	comments = NULL;
 	return XcursorFalse;
     }
     *imagesp = images;
@@ -818,7 +816,7 @@ XcursorXcFileLoad (XcursorFile	    *file,
 }
 
 XcursorBool
-XcursorXcFileSave (XcursorFile		    *file, 
+XcursorXcFileSave (XcursorFile		    *file,
 		   const XcursorComments    *comments,
 		   const XcursorImages	    *images)
 {
@@ -826,21 +824,21 @@ XcursorXcFileSave (XcursorFile		    *file,
     XcursorUInt		position;
     int			n;
     int			toc;
-    
+
     if (!file || !comments || !images)
         return XcursorFalse;
-    
+
     fileHeader = _XcursorFileHeaderCreate (comments->ncomment + images->nimage);
     if (!fileHeader)
 	return XcursorFalse;
-    
+
     position = _XcursorFileHeaderLength (fileHeader);
 
     /*
      * Compute the toc.  Place the images before the comments
      * as they're more often read
      */
-    
+
     toc = 0;
     for (n = 0; n < images->nimage; n++)
     {
@@ -850,7 +848,7 @@ XcursorXcFileSave (XcursorFile		    *file,
 	position += _XcursorImageLength (images->images[n]);
 	toc++;
     }
-    
+
     for (n = 0; n < comments->ncomment; n++)
     {
 	fileHeader->tocs[toc].type = XCURSOR_COMMENT_TYPE;
@@ -859,13 +857,13 @@ XcursorXcFileSave (XcursorFile		    *file,
 	position += _XcursorCommentLength (comments->comments[n]);
 	toc++;
     }
-    
+
     /*
      * Write the header and the toc
      */
     if (!_XcursorWriteFileHeader (file, fileHeader))
 	goto bail;
-    
+
     /*
      * Write the images
      */
@@ -876,7 +874,7 @@ XcursorXcFileSave (XcursorFile		    *file,
 	    goto bail;
 	toc++;
     }
-    
+
     /*
      * Write the comments
      */
@@ -886,7 +884,7 @@ XcursorXcFileSave (XcursorFile		    *file,
 	    goto bail;
 	toc++;
     }
-    
+
     _XcursorFileHeaderDestroy (fileHeader);
     return XcursorTrue;
 bail:
@@ -961,8 +959,8 @@ XcursorFileLoadAllImages (FILE *file)
 }
 
 XcursorBool
-XcursorFileLoad (FILE		    *file, 
-		 XcursorComments    **commentsp, 
+XcursorFileLoad (FILE		    *file,
+		 XcursorComments    **commentsp,
 		 XcursorImages	    **imagesp)
 {
     XcursorFile	f;
@@ -977,10 +975,13 @@ XcursorFileLoad (FILE		    *file,
 XcursorBool
 XcursorFileSaveImages (FILE *file, const XcursorImages *images)
 {
-    XcursorComments *comments = XcursorCommentsCreate (0);
+    XcursorComments *comments;
     XcursorFile	    f;
     XcursorBool	    ret;
-    if (!comments || !file || !images)
+
+    if (!file || !images)
+	return 0;
+    if ((comments = XcursorCommentsCreate (0)) == NULL)
 	return 0;
     _XcursorStdioFileInitialize (file, &f);
     ret = XcursorXcFileSave (&f, comments, images) && fflush (file) != EOF;
@@ -989,7 +990,7 @@ XcursorFileSaveImages (FILE *file, const XcursorImages *images)
 }
 
 XcursorBool
-XcursorFileSave (FILE *			file, 
+XcursorFileSave (FILE *			file,
 		 const XcursorComments	*comments,
 		 const XcursorImages	*images)
 {
@@ -997,7 +998,7 @@ XcursorFileSave (FILE *			file,
 
     if (!file || !comments || !images)
         return XcursorFalse;
-    
+
     _XcursorStdioFileInitialize (file, &f);
     return XcursorXcFileSave (&f, comments, images) && fflush (file) != EOF;
 }
@@ -1013,7 +1014,7 @@ XcursorFilenameLoadImage (const char *file, int size)
 
     f = fopen (file, "r");
     if (!f)
-	return 0;
+	return NULL;
     image = XcursorFileLoadImage (f, size);
     fclose (f);
     return image;
@@ -1027,10 +1028,10 @@ XcursorFilenameLoadImages (const char *file, int size)
 
     if (!file || size < 0)
         return NULL;
-    
+
     f = fopen (file, "r");
     if (!f)
-	return 0;
+	return NULL;
     images = XcursorFileLoadImages (f, size);
     fclose (f);
     return images;
@@ -1047,7 +1048,7 @@ XcursorFilenameLoadAllImages (const char *file)
 
     f = fopen (file, "r");
     if (!f)
-	return 0;
+	return NULL;
     images = XcursorFileLoadAllImages (f);
     fclose (f);
     return images;
@@ -1089,7 +1090,7 @@ XcursorFilenameSaveImages (const char *file, const XcursorImages *images)
 }
 
 XcursorBool
-XcursorFilenameSave (const char		    *file, 
+XcursorFilenameSave (const char		    *file,
 		     const XcursorComments  *comments,
 		     const XcursorImages    *images)
 {
